/**
 * 
 * Copyright (c) 2006-2018, Speedment, Inc. All Rights Reserved.
 * 
 * Licensed under the Apache License, Version 2.0 (the "License"); You may not
 * use this file except in compliance with the License. You may obtain a copy of
 * the License at: 
 * 
 * http://www.apache.org/licenses/LICENSE-2.0
 * 
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package com.speedment.runtime.field;

import com.speedment.common.annotation.GeneratedCode;
import com.speedment.runtime.compute.ToChar;
import com.speedment.runtime.config.identifier.ColumnIdentifier;
import com.speedment.runtime.field.comparator.CharFieldComparator;
import com.speedment.runtime.field.internal.CharFieldImpl;
import com.speedment.runtime.field.method.CharGetter;
import com.speedment.runtime.field.method.CharSetter;
import com.speedment.runtime.field.trait.HasCharValue;
import com.speedment.runtime.field.trait.HasComparableOperators;
import com.speedment.runtime.typemapper.TypeMapper;

/**
 * A field that represents a primitive {@code char} value.
 * <p>
 * Generated by com.speedment.sources.pattern.FieldPattern.
 * 
 * @param <ENTITY> entity type
 * @param <D>      database type
 * 
 * @author Emil Forslund
 * @since  3.0.0
 */
@GeneratedCode(value = "Speedment")
public interface CharField<ENTITY, D> extends Field<ENTITY>, HasCharValue<ENTITY, D>, HasComparableOperators<ENTITY, Character>, ToChar<ENTITY>, CharFieldComparator<ENTITY, D> {
    
    /**
     * Creates a new {@link CharField} using the default implementation.
     * 
     * @param <ENTITY>   entity type
     * @param <D>        database type
     * @param identifier column that this field represents
     * @param getter     method reference to getter in entity
     * @param setter     method reference to setter in entity
     * @param typeMapper type mapper that is applied
     * @param unique     if column only contains unique values
     * @return           the created field
     */
    static <ENTITY, D> CharField<ENTITY, D> create(
    ColumnIdentifier<ENTITY> identifier,
            CharGetter<ENTITY> getter,
            CharSetter<ENTITY> setter,
            TypeMapper<D, Character> typeMapper,
            boolean unique) {
        return new CharFieldImpl<>(
            identifier, getter, setter, typeMapper, unique
        );
    }
    
    @Override
    CharField<ENTITY, D> tableAlias(String tableAlias);
    
    @Override
    default char applyAsChar(ENTITY entity) {
        return getAsChar(entity);
    }
    
    @Override
    CharFieldComparator<ENTITY, D> comparator();
    
    @Override
    default CharFieldComparator<ENTITY, D> reversed() {
        return comparator().reversed();
    }
    
    @Override
    default CharField<ENTITY, D> getField() {
        return this;
    }
}